#include "StFemtoEvent.h"
#include <TMath.h>

ClassImp(StFemtoEvent)

//_________________
StFemtoEvent::StFemtoEvent() :
  mEventId(0), mRunId(0), mCollisionType(false), mRefMult(0), mRefMultCorr(0),
  mRefMultCorrWeight(0), mRefMultPos(0), mRefMult2(0),
  mRefMult2Pos(0), mGRefMult(0), mCent16(0), mNumberOfPrimaryTracks(0),
  mNumberOfGlobalTracks(0), mNumberOfBTofHit(0), mNumberOfTofMatched(0),
  mMagField(0), mVertexPositionX(-999.), mVertexPositionY(-999.), mVertexPositionZ(-999.),
  mVpdVz(0), mPrimaryVertexRanking(-9), mNumberOfPrimaryVertices(0),
  mTransverseSphericity(0), mTransverseSphericity2(0), mZdcSumAdcEast(0), mZdcSumAdcWest(0),
  mZdcSmdEastHorizontal{}, mZdcSmdEastVertical{}, mZdcSmdWestHorizontal{}, mZdcSmdWestVertical{},
  mBbcAdcEast{}, mBbcAdcWest{} {

  mFemtoTracks = new TClonesArray("StFemtoTrack", 1500);
  mNFemtoTracks = 0;
}

//_________________
StFemtoEvent::StFemtoEvent(const StFemtoEvent& ev) {
  
  mEventId = ev.mEventId;
  mRunId = ev.mRunId;
  mCollisionType = ev.mCollisionType;
  mRefMult = ev.mRefMult;
  mRefMultCorr = ev.mRefMultCorr;
  mRefMultCorrWeight = ev.mRefMultCorrWeight;
  mRefMultPos = ev.mRefMultPos;
  mGRefMult = ev.mGRefMult;
  mCent16 = ev.mCent16;
  mNumberOfPrimaryTracks = ev.mNumberOfPrimaryTracks;
  mNumberOfGlobalTracks = ev.mNumberOfGlobalTracks;
  mNumberOfBTofHit = ev.mNumberOfBTofHit;
  mNumberOfTofMatched = ev.mNumberOfTofMatched;
  mMagField = ev.mMagField;
  mVertexPositionX = ev.mVertexPositionX;
  mVertexPositionY = ev.mVertexPositionY;
  mVertexPositionZ = ev.mVertexPositionZ;
  mVpdVz = ev.mVpdVz;
  mPrimaryVertexRanking = ev.mPrimaryVertexRanking;
  mNumberOfPrimaryVertices = ev.mNumberOfPrimaryVertices;
  mTransverseSphericity = ev.mTransverseSphericity;
  mTransverseSphericity2 = ev.mTransverseSphericity2;
  
  mZdcSumAdcEast = ev.mZdcSumAdcEast;
  mZdcSumAdcWest = ev.mZdcSumAdcWest;
  for(int iModule=0; iModule<8; iModule++) {
    mZdcSmdEastHorizontal[iModule] = ev.mZdcSmdEastHorizontal[iModule];
    mZdcSmdEastVertical[iModule] = ev.mZdcSmdEastVertical[iModule];
    mZdcSmdWestHorizontal[iModule] = ev.mZdcSmdWestHorizontal[iModule];
    mZdcSmdWestVertical[iModule] = ev.mZdcSmdWestVertical[iModule];
  }

  for(int iModule=0; iModule<24; iModule++) {
    mBbcAdcEast[iModule] = ev.mBbcAdcEast[iModule];
    mBbcAdcWest[iModule] = ev.mBbcAdcWest[iModule];
  }

  for(unsigned int iIter=0; ev.mTriggerIds.size(); iIter++) {
    mTriggerIds.at(iIter) = ev.mTriggerIds.at(iIter);
  }

  mNFemtoTracks = ev.mNFemtoTracks;
  mFemtoTracks = new TClonesArray(*ev.mFemtoTracks);
}

//_________________
StFemtoEvent::~StFemtoEvent() {
  /* nothing */
}

//_________________
void StFemtoEvent::Clear(Option_t *option) {

  mFemtoTracks->Clear(option);
  mNFemtoTracks = 0;
}

//_________________
Int_t StFemtoEvent::GetYear() const {
  
  return (mRunId / 1000000) - 1 + 2000;
}

//_________________
Int_t StFemtoEvent::GetDay() const {
  
  return (mRunId % 1000000) / 1000;
}

//_________________
void StFemtoEvent::SetNumberOfPrimaryVertices(Int_t mult) {
  
  if(mult<0 || mult>100) {
    mNumberOfPrimaryVertices = 0;
  }
  else {
    mNumberOfPrimaryVertices = (UChar_t)mult;
  }
}

//_________________
Bool_t StFemtoEvent::IsTrigger(unsigned int id) const {
  
  return std::find(mTriggerIds.begin(), mTriggerIds.end(), id) != mTriggerIds.end();
}

//_________________
StFemtoTrack* StFemtoEvent::AddFemtoTrack() {

  TClonesArray &tracks = *mFemtoTracks;
  StFemtoTrack *track = new(tracks[mNFemtoTracks++]) StFemtoTrack;
  return track;
}

//_________________
void StFemtoEvent::SetPrimaryVertexPosition(Float_t x, Float_t y, Float_t z) { 

  mVertexPositionX = x;
  mVertexPositionY = y;
  mVertexPositionZ = z;
}

//_________________
TVector3 StFemtoEvent::GetPrimaryVertexPosition() const { 

  TVector3 p(mVertexPositionX, mVertexPositionY, mVertexPositionZ);
  return p; 
}

//_________________
Float_t StFemtoEvent::GetVertexPositionR() const {

  Float_t r = TMath::Sqrt(mVertexPositionX*mVertexPositionX + 
			  mVertexPositionY*mVertexPositionY);
  return r;
}

//_________________
void StFemtoEvent::SetPrimaryVertexRanking(Float_t rank) {

  if(rank <= 0.) { 
    mPrimaryVertexRanking = -9;
  }
  else if(rank <= 1.) {
    mPrimaryVertexRanking = 1;
  }
  else if(rank <= 100) {
    mPrimaryVertexRanking = 2;
  }
  else if(rank <= 1e9) {
    mPrimaryVertexRanking = 3;
  }
  else {
    mPrimaryVertexRanking = 4;
  }
}

//_________________
void StFemtoEvent::SetRefMult(Int_t mult) {

  if(mult<0 || mult>60000) {
    mRefMult = USHORTMAX;
  }
  else {
    mRefMult = (UShort_t)mult;
  }
}

//_________________
void StFemtoEvent::SetGRefMult(Int_t mult) {
  
  if(mult<0 || mult>60000) {
    mGRefMult = USHORTMAX;
  }
  else {
    mGRefMult = (UShort_t)mult;
  }
}

//_________________
void StFemtoEvent::SetNumberOfTofMatched(Int_t mult) {
  
  if(mult<0 || mult>60000) {
    mNumberOfTofMatched = USHORTMAX;
  }
  else {
    mNumberOfTofMatched = (UShort_t)mult;
  }  
}

//_________________
void StFemtoEvent::SetRefMult2(Int_t mult) {

  if(mult<0 || mult>60000) {
    mRefMult2 = USHORTMAX;
  }
  else {
    mRefMult2 = (UShort_t)mult;
  }
}

//_________________
void StFemtoEvent::SetRefMultCorr(Float_t rf) {

  if(rf<0. || rf >6000) {
    mRefMultCorr = USHORTMAX;
  }
  else {
    mRefMultCorr = (UShort_t)(rf * 10);
  }
}

//_________________
void StFemtoEvent::SetTransverseSphericity(Float_t sph) {
  
  if(sph<0. || sph>1.) {
    mTransverseSphericity = (UChar_t)(sph * 100.);
  }
  else {
    mTransverseSphericity = 1.;
  }
}

//_________________
void StFemtoEvent::SetTransverseSphericity2(Float_t sph) {
  
  if(sph<0. || sph>1.) {
    mTransverseSphericity2 = (UChar_t)(sph * 100.);
  }
  else {
    mTransverseSphericity2 = 1.;
  }
}

//_________________
void StFemtoEvent::SetRefMultCorrWeight(Float_t w) {

  if(w<0. || w>6.) {
    mRefMultCorrWeight = USHORTMAX;
  }
  else {
    mRefMultCorrWeight = (UShort_t)(w * 10000);
  }
}

//_________________
void StFemtoEvent::SetMagneticField(Float_t f) {

  if(TMath::Abs(f*1000) > SHORTMAX) {
    if(f>0) {
      mMagField = SHORTMAX;
    }
    else {
      mMagField = -SHORTMAX;
    }
  }
  else {
    mMagField = (Short_t)(f*1000);
  }
}

//_________________
UShort_t StFemtoEvent::GetCent9() const {

  if(mCent16 == 15) {  
    return 8;  //0-5%      
  }
  else if(mCent16 == 14) {
    return 7;  //5-10%
  }
  else if( (mCent16 == 13) || (mCent16 == 12) ) {
    return 6;  //10-20%
  }
  else if( (mCent16 == 11) || (mCent16 == 10) ) {
    return 5;  //20-30%
  }
  else if( (mCent16 == 9) || (mCent16 == 8) ) {
    return 4;  //30-40%
  }
  else if( (mCent16 == 7) || (mCent16 == 6) ) {
    return 3;  //40-50%
  }
  else if( (mCent16 == 5) || (mCent16 == 4) ) {
    return 2;  //50-60%
  }
  else if( (mCent16 == 3) || (mCent16 == 2) ) {
    return 1;  //60-70%
  }
  else if( (mCent16 == 1) || (mCent16 == 0) ) {
    return 0;  //70-80%
  }
  else {
    return 9999;
  }
}

//_________________
void StFemtoEvent::SetZdcSumAdcEast(Float_t x) {

  if(x<0 || x>USHORTMAX) {
    mZdcSumAdcEast = USHORTMAX;
  }
  else {
    mZdcSumAdcEast = (UShort_t)x;
  }
}

//_________________
void StFemtoEvent::SetZdcSumAdcWest(Float_t x) {

  if(x<0 || x>USHORTMAX) {
    mZdcSumAdcWest = USHORTMAX;
  }
  else {
    mZdcSumAdcWest = (UShort_t)x;
  }
}

//_________________
void StFemtoEvent::SetZdcSmdEastHorizontal(Int_t iMod, Float_t x) {
  
  if(x<0 || x>USHORTMAX) {
    mZdcSmdEastHorizontal[iMod] = USHORTMAX;
  }
  else {
    mZdcSmdEastHorizontal[iMod] = (UShort_t)x;
  }
}

//_________________
void StFemtoEvent::SetZdcSmdEastVertical(Int_t iMod, Float_t x) {
  
  if(x<0 || x>USHORTMAX) {
    mZdcSmdEastVertical[iMod] = USHORTMAX;
  }
  else {
    mZdcSmdEastVertical[iMod] = (UShort_t)x;
  }
}

//_________________
void StFemtoEvent::SetZdcSmdWestHorizontal(Int_t iMod, Float_t x) {
  
  if(x<0 || x>USHORTMAX) {
    mZdcSmdWestHorizontal[iMod] = USHORTMAX;
  }
  else {
    mZdcSmdWestHorizontal[iMod] = (UShort_t)x;
  }
}

//_________________
void StFemtoEvent::SetZdcSmdWestVertical(Int_t iMod, Float_t x) {
  
  if(x<0 || x>USHORTMAX) {
    mZdcSmdWestVertical[iMod] = USHORTMAX;
  }
  else {
    mZdcSmdWestVertical[iMod] = (UShort_t)x;
  }
}

//_________________
void StFemtoEvent::SetBbcAdcEast(Int_t iMod, UShort_t x) {
  
  if(x<0 || x>USHORTMAX) {
    mBbcAdcEast[iMod] = USHORTMAX;
  }
  else {
    mBbcAdcEast[iMod] = (UShort_t)x;
  }
}

//_________________
void StFemtoEvent::SetBbcAdcWest(Int_t iMod, UShort_t x) {
  
  if(x<0 || x>USHORTMAX) {
    mBbcAdcWest[iMod] = USHORTMAX;
  }
  else {
    mBbcAdcWest[iMod] = (UShort_t)x;
  }
}

//_________________
void StFemtoEvent::SetTriggerId(UInt_t id) {

  //Checking if the trigger id is already in the vector mTriggerIds
  if ( std::find(mTriggerIds.begin(),
                 mTriggerIds.end(),
                 id) != mTriggerIds.end() ) {

    mTriggerIds.push_back(id);
  }
}

//_________________
void StFemtoEvent::SetTrigger(UInt_t id) {
  SetTrigger(id);
}

//_________________
void StFemtoEvent::AddTriggerId(UInt_t id) {
  SetTriggerId(id);
}
