//
//  The version of the class should be changed every time
//  when any changes in the codes are done
//  Grigory Nigmatkulov: 2016/12/15
//
#ifndef STFEMTOEVENT_H
#define STFEMTOEVENT_H

#include <TObject.h>
#include <TClonesArray.h>
#include <TVector3.h>

#include "StFemtoTrack.h"

#include <vector>

//_________________
class StFemtoEvent : public TObject {

 public:

  //Constructor
  StFemtoEvent();
  //Copy constructor
  StFemtoEvent(const StFemtoEvent &copy);
  //Destructor
  virtual ~StFemtoEvent();

  //Setters
  void SetEventId(Int_t id)            { mEventId = id; }
  void SetRunId(Int_t id)              { mRunId = id; }
  void SetCollisionType(Bool_t type)   { mCollisionType = type;}
  
  void SetRefMult(Int_t mult);
  void SetRefMult2(Int_t mult);
  void SetRefMultCorr(Float_t rf);
  void SetRefMultCorrWeight(Float_t w);
  void SetRefMultPos(Int_t m)          { mRefMultPos = (UShort_t)m; }
  void SetRefMult2Pos(Int_t m)         { mRefMult2Pos = (UShort_t)m; }
  void SetGRefMult(Int_t m);
  void SetCent16(Int_t c)              { mCent16 = (Char_t)c; }
  void SetNumberOfBTofHit(Int_t m)     { mNumberOfBTofHit = (m<=0) ? 0 : (UShort_t)m; }
  void SetNumberOfBTofHit(UInt_t m)    { mNumberOfBTofHit = (m<=0) ? 0 : (UShort_t)m; }
  void SetNumberOfTofMatched(Int_t m);
  void SetNumberOfPrimaryTracks(Int_t m) { mNumberOfPrimaryTracks = (UShort_t)m; }
  void SetNumberOfGlobalTracks(Int_t m)  { mNumberOfGlobalTracks = (UShort_t)m; }

  void SetZdcSumAdcEast(Float_t x);
  void SetZdcSumAdcWest(Float_t x);
  void SetZdcSmdEastHorizontal(Int_t i, Float_t x);
  void SetZdcSmdEastVertical(Int_t i, Float_t x);
  void SetZdcSmdWestHorizontal(Int_t i, Float_t x);
  void SetZdcSmdWestVertical(Int_t i, Float_t x);
  void SetBbcAdcEast(Int_t i, UShort_t x);
  void SetBbcAdcWest(Int_t i, UShort_t x);
  
  void SetMagneticField(Float_t f);
  void SetPrimaryVertexPosition(Float_t x, Float_t y, Float_t z);
  void SetPrimaryVertexX(Float_t x)   { mVertexPositionX = x; }
  void SetPrimaryVertexY(Float_t y)   { mVertexPositionY = y; }
  void SetPrimaryVertexZ(Float_t z)   { mVertexPositionZ = z; }
  void SetNumberOfPrimaryVertices(Int_t mult);
  void SetTransverseSphericity(Float_t sph);
  void SetTransverseSphericity2(Float_t sph);
  void SetVpdVz(Float_t vz)           { mVpdVz = vz; }
  void SetPrimaryVertexRanking(Float_t);
  void AddTriggerId(UInt_t id);
  void SetTriggerId(UInt_t id);
  void SetTrigger(UInt_t id);
  StFemtoTrack* AddFemtoTrack();
  void Clear(Option_t* option = "C");

  //Getters
  Int_t    GetEventId() const          { return mEventId; }
  Int_t    GetRunId() const            { return mRunId; }
  Int_t    GetYear() const;
  Int_t    GetDay() const;
  Bool_t   GetCollisionType() const    { return mCollisionType; }
  UInt_t   GetRefMult() const          { return (UInt_t)mRefMult; }
  UInt_t   GetRefMult2() const         { return (UInt_t)mRefMult2;}
  Float_t  GetRefMultCorr() const      { return (Float_t)mRefMultCorr * 0.1; }
  Float_t  GetRefMultCorrWeight() const{ return (Float_t)mRefMultCorrWeight * 0.0001; }
  UInt_t   GetGRefMult() const         { return (UInt_t)mGRefMult; }
  UShort_t GetCent9() const;
  UShort_t GetCent16() const           { return (UShort_t)mCent16; }
  UShort_t GetRefMultPos() const       { return mRefMultPos; }
  UShort_t GetRefMultNeg() const       { return (mRefMult - mRefMultPos); }
  UShort_t GetRefMult2Pos() const      { return mRefMult2Pos; }
  UShort_t GetRefMult2Neg() const      { return (mRefMult2 - mRefMult2Pos); }

  Float_t  GetZdcSumAdcEast() const    { return (Float_t)mZdcSumAdcEast; }
  Float_t  GetZdcSumAdcWest() const    { return (Float_t)mZdcSumAdcWest; }
  Float_t  GetZdcSmdEastHorizontal(Int_t i) const { return (Float_t)mZdcSmdEastHorizontal[i]; }
  Float_t  GetZdcSmdEastVertical(Int_t i) const   { return (Float_t)mZdcSmdEastVertical[i]; }
  Float_t  GetZdcSmdWestHorizontal(Int_t i) const { return (Float_t)mZdcSmdWestHorizontal[i]; }
  Float_t  GetZdcSmdWestVertical(Int_t i) const   { return (Float_t)mZdcSmdWestVertical[i]; }
  UShort_t GetBbcAdcEast(Int_t i) const           { return mBbcAdcEast[i]; }
  UShort_t GetBbcAdcWest(Int_t i) const           { return mBbcAdcWest[i]; }
 
  UShort_t GetNumberOfBTofHit() const       { return mNumberOfBTofHit; }
  UShort_t GetNumberOfPrimaryTracks() const { return mNumberOfPrimaryTracks; }
  UShort_t GetNumberOfGlobalTracks() const  { return mNumberOfGlobalTracks; }
  UShort_t GetNumberOfTofMatched() const    { return mNumberOfTofMatched; }
  Float_t  GetMagneticField() const         { return (Float_t)mMagField*0.001; }
  TVector3 GetPrimaryVertexPosition() const;
  Float_t  GetVertexPositionX() const       { return mVertexPositionX; }
  Float_t  GetVertexPositionY() const       { return mVertexPositionY; }
  Float_t  GetVertexPositionZ() const       { return mVertexPositionZ; }
  UInt_t   GetNumberOfPrimaryVertices() const { return (UInt_t)mNumberOfPrimaryVertices; }
  Float_t  GetTransverseSphericity() const  { return (Float_t)mTransverseSphericity * 0.01; }
  Float_t  GetTransverseSphericity2() const { return (Float_t)mTransverseSphericity2 * 0.01; }
  Float_t  GetVertexPositionR() const;
  Float_t  GetVpdVz() const                 { return mVpdVz; }

  std::vector<unsigned int> GetTriggerIds() const { return mTriggerIds; }
  Bool_t   IsTrigger(unsigned int id) const;
  
  Float_t  GetPrimaryVertexRanking() const  { return (Float_t)mPrimaryVertexRanking; }
  TClonesArray *GetFemtoTracks() const      { return mFemtoTracks; }
  Int_t    GetNFemtoTracks() const          { return (Int_t)mNFemtoTracks; }

 private:
  
  Int_t    mEventId;
  Int_t    mRunId;
  Bool_t   mCollisionType;       //0-pp, 1-AA(from those where RefMultCorr is defined)

  UShort_t mRefMult;             //mRefMult
  UShort_t mRefMultCorr;         //mRefMultCorr * 10
  UShort_t mRefMultCorrWeight;   //mRefMultCorrWight * 10000
  UShort_t mRefMultPos;          //n positive for refMult
  UShort_t mRefMult2;            //refMult for |eta| < 1.0
  UShort_t mRefMult2Pos;         //n positive for refMult2
  UShort_t mGRefMult;            //mGRefMult
  Char_t   mCent16;

  UShort_t mNumberOfPrimaryTracks;  //in event for all vertices
  UShort_t mNumberOfGlobalTracks;   //total number of global tracks in event
  UShort_t mNumberOfBTofHit;        //btofTrayMultiplicity
  UShort_t mNumberOfTofMatched;     //number of tof matched tracks
  
  Short_t  mMagField;               //Field*1000
  Float_t  mVertexPositionX;        //VtxX
  Float_t  mVertexPositionY;        //VtxY
  Float_t  mVertexPositionZ;        //VtxZ
  Float_t  mVpdVz;                  //VzVpd

  Char_t   mPrimaryVertexRanking;    //-9: bad; 1: <1; 2: <100; 3: <10^9; 4: > 10^9
  UChar_t  mNumberOfPrimaryVertices; //Number of primary vertices in the event
  UChar_t  mTransverseSphericity;    //Transverse sphericity: pTrk with p>0.15 GeV, |eta|<0.5, nHits>15
  UChar_t  mTransverseSphericity2;   //Transverse sphericity: pTrk with p>0.15 GeV, |eta|<1, nHits>15
  
  std::vector<unsigned int> mTriggerIds;

  //ZDC information
  UShort_t mZdcSumAdcEast;
  UShort_t mZdcSumAdcWest;
  UShort_t mZdcSmdEastHorizontal[8];
  UShort_t mZdcSmdEastVertical[8];
  UShort_t mZdcSmdWestHorizontal[8];
  UShort_t mZdcSmdWestVertical[8];

  // BBC ADC for q-vectors (Hiroshi)
  UShort_t mBbcAdcEast[24];
  UShort_t mBbcAdcWest[24];
  
  UShort_t mNFemtoTracks;
  TClonesArray *mFemtoTracks;

  ClassDef(StFemtoEvent, 4)
};

#endif
