#include "StFemtoTrack.h"

ClassImp(StFemtoTrack)

//_________________
StFemtoTrack::StFemtoTrack() : mId(0), mNHits(0), mNHitsPoss(0),
  mNSigmaElectron(-30.), mNSigmaPion(-30.), mNSigmaKaon(-30.), mNSigmaProton(-30.),
  mDedx(0), mMap0(0), mMap1(0), mMomX(0.), mMomY(0.), mMomZ(0.),
  mOriginX(0), mOriginY(0), mOriginZ(0), mGlobMomX(0.), mGlobMomY(0.), mGlobMomZ(0.),
  mBeta(0) {
  /* empty */
}

//_________________
StFemtoTrack::StFemtoTrack(const StFemtoTrack &t) {

  //Copy constructor
  mId = t.mId;
  mNHits = t.mNHits;
  mNHitsPoss = t.mNHitsPoss;
  mNSigmaElectron = t.mNSigmaElectron;
  mNSigmaPion = t.mNSigmaPion;
  mNSigmaKaon = t.mNSigmaKaon;
  mNSigmaProton = t.mNSigmaProton;
  mDedx = t.mDedx;
  mMap0 = t.mMap0;
  mMap1 = t.mMap1;
  mMomX = t.mMomX; mMomY = t.mMomY; mMomZ = t.mMomZ;
  mOriginX = t.mOriginX; mOriginY = t.mOriginY; mOriginZ = t.mOriginZ;
  mGlobMomX = t.mGlobMomX; mGlobMomY = t.mGlobMomY; mGlobMomZ = t.mGlobMomZ;
  mBeta = t.mBeta;
}

//_________________
StFemtoTrack::~StFemtoTrack() {
  /* empty */
}

//_________________
Short_t StFemtoTrack::GetType() const { 

  //Get type of the track: global, primary
  if( (mMomX <= 1e-5) &&
      (mMomY <= 1e-5) &&
      (mMomZ <= 1e-5) ) {
    return 0;            //Global
  }
  else {
    return 1;            //Primary
  }
}

//_________________
Bool_t StFemtoTrack::GetIsPrimary() const {
  return GetType();
}

//_________________
Bool_t StFemtoTrack::GetIsTofTrack() { 

  //Checks if track was matched to TOF
  if(mBeta <= 0) {
    return false;
  }
  else {
    return true;
  }
}

//_________________
Float_t StFemtoTrack::GetEta() const {

  Float_t ptot = TMath::Sqrt(mMomX*mMomX + mMomY*mMomY + mMomZ*mMomZ);
  Float_t eta = 0.5*TMath::Log( (ptot + mMomZ) / (ptot - mMomZ) );
  return eta;
}


//_________________
TVector3 StFemtoTrack::GetP() const {

  //Return 3-momentum of the primary track
  TVector3 p(mMomX, mMomY, mMomZ);
  return p;
}

//_________________
Float_t StFemtoTrack::GetPt() const {

  //Return transverse momentum of the primary track
  return TMath::Sqrt( mMomX * mMomX +
		      mMomY * mMomY );
}

//_________________
Float_t StFemtoTrack::GetPtot() const {

  //Return momentum of the primary track
  return TMath::Sqrt( mMomX * mMomX +
		      mMomY * mMomY +
		      mMomZ * mMomZ );
}

//_________________
TVector3 StFemtoTrack::GetGlobalP() const {

  //Return 3-vector of the global track
  TVector3 p( mGlobMomX, mGlobMomY, mGlobMomZ );
  return p;
}

//_________________
Float_t StFemtoTrack::GetGlobalPt() const {

  //Return transverse momentum of the global track
  return TMath::Sqrt(mGlobMomX*mGlobMomX + mGlobMomY*mGlobMomY);
}

//_________________
Float_t StFemtoTrack::GetGlobalPtot() const {

  //Return momentum of the global track
  return TMath::Sqrt( mGlobMomX * mGlobMomX +
		      mGlobMomY * mGlobMomY +
		      mGlobMomZ * mGlobMomZ );
}

//_________________
Float_t StFemtoTrack::GetMassSqr() const {

  //Calculate squared mass of the track via TOF beta measurement
  Float_t mSqr = -999.;
  Float_t beta = GetBeta();
  if(mBeta > 0.) {
    Float_t ptot = TMath::Sqrt(mMomX*mMomX + mMomY*mMomY + mMomZ*mMomZ);
    mSqr = ptot*ptot * (1./(beta*beta) - 1.);
  }

  return mSqr;
}

//_________________
void StFemtoTrack::SetNSigmaElectron(Float_t ns) {

  if(TMath::Abs(ns*__SCALE__) >= SHORTMAX) {
    if(ns>0) {
      mNSigmaElectron = SHORTMAX;
    }
    else {
      mNSigmaElectron = -SHORTMAX;
    }
  }
  else {
    mNSigmaElectron = (Short_t)(ns*__SCALE__);
  }
}

//_________________
void StFemtoTrack::SetNSigmaPion(Float_t ns) {

  if(TMath::Abs(ns*__SCALE__) >= SHORTMAX) {
    if(ns>0) {
      mNSigmaPion = SHORTMAX;
    }
    else {
      mNSigmaPion = -SHORTMAX;
    }
  }
  else {
    mNSigmaPion = (Short_t)(ns*__SCALE__);
  }
}

//_________________
void StFemtoTrack::SetNSigmaKaon(Float_t ns) {

  if(TMath::Abs(ns*__SCALE__) >= SHORTMAX) {
    if(ns>0) {
      mNSigmaKaon = SHORTMAX;
    }
    else {
      mNSigmaKaon = -SHORTMAX;
    }
  }
  else {
    mNSigmaKaon = (Short_t)(ns*__SCALE__);
  }
}

//_________________
void StFemtoTrack::SetNSigmaProton(Float_t ns) {

  if(TMath::Abs(ns*__SCALE__) >= SHORTMAX) {
    if(ns>0) {
      mNSigmaProton = SHORTMAX;
    }
    else {
      mNSigmaProton = -SHORTMAX;
    }
  }
  else {
    mNSigmaProton = (Short_t)(ns*__SCALE__);
  }
}

//_________________
void StFemtoTrack::SetDedx(Float_t dEdx) {

  if(dEdx >= USHORTMAX) {
    mDedx = USHORTMAX;
  }
  else {
    mDedx = (UShort_t)(dEdx * 1e9);
  }
}

//_________________
void StFemtoTrack::SetDedx(Double_t dEdx) {

  if(dEdx >= USHORTMAX) {
    mDedx = USHORTMAX;
  }
  else {
    mDedx = (UShort_t)(dEdx * 1e9);
  }
}

//_________________
Float_t StFemtoTrack::GetGlobalDCAx(Float_t pVtxX) const {

  //Calculates DCA x of the global track to pVtx
  return (mOriginX - pVtxX);
}

//_________________
Float_t StFemtoTrack::GetGlobalDCAy(Float_t pVtxY) const {

  //Calculates DCA y of the global track to pVtx
  return (mOriginY - pVtxY);
}

//_________________
Float_t StFemtoTrack::GetGlobalDCAz(Float_t pVtxZ) const {

  //Calculates DCA z of the global track to pVtx
  return (mOriginZ - pVtxZ);
}

//_________________
Float_t StFemtoTrack::GetGlobalDCAxy(Float_t pVtxX, Float_t pVtxY) const {

  //Calculates DCA perp of the global track to pVtx
  return TMath::Sqrt( GetGlobalDCAx(pVtxX) * GetGlobalDCAx(pVtxX) +
		      GetGlobalDCAy(pVtxY) * GetGlobalDCAy(pVtxY) );
}

//_________________
Float_t StFemtoTrack::GetGlobalDCAxyz(Float_t pVtxX, Float_t pVtxY, Float_t pVtxZ) const {

  //Calculates DCA of the global track to pVtx
  return TMath::Sqrt( GetGlobalDCAx(pVtxX) * GetGlobalDCAx(pVtxX) +
		      GetGlobalDCAy(pVtxY) * GetGlobalDCAy(pVtxY) +
		      GetGlobalDCAz(pVtxZ) * GetGlobalDCAz(pVtxZ) );
}

//_________________
Float_t StFemtoTrack::GetDCAx(Float_t pVtxX) const {

  //Return DCA of the primary track
  if( GetIsPrimary() ) {
    return 0.;  //Primary tracks have 0 dca to primary vertex
  }
  else {
    return GetGlobalDCAx(pVtxX);
  }
}

//_________________
Float_t StFemtoTrack::GetDCAy(Float_t pVtxY) const {

  //Return DCA of the primary track
  if( GetIsPrimary() ) {
    return 0.;  //Primary tracks have 0 dca to primary vertex
  }
  else {
    return GetGlobalDCAx(pVtxY);
  }
}

//_________________
Float_t StFemtoTrack::GetDCAz(Float_t pVtxZ) const {

  //Return DCA of the primary track
  if( GetIsPrimary() ) {
    return 0.;  //Primary tracks have 0 dca to primary vertex
  }
  else {
    return GetGlobalDCAz(pVtxZ);
  }
}

//_________________
Float_t StFemtoTrack::GetDCAxy(Float_t pVtxX, Float_t pVtxY) const {

  //Return DCA of the primary track
  if( GetIsPrimary() ) {
    return 0.;  //Primary tracks have 0 dca to primary vertex
  }
  else {
    return GetGlobalDCAxy(pVtxX, pVtxY);
  }
}

//_________________
Float_t StFemtoTrack::GetDCAxyz(Float_t pVtxX, Float_t pVtxY, Float_t pVtxZ) const {

  //Return DCA of the primary track
  if( GetIsPrimary() ) {
    return 0.;  //Primary tracks have 0 dca to primary vertex
  }
  else {
    return GetGlobalDCAxyz(pVtxX, pVtxY, pVtxZ);
  }
}

//_________________
void StFemtoTrack::SetBeta(Float_t beta) {
  if(beta<0.) {
    mBeta = 0;
  }
  else {
    mBeta = ( (beta * 20000.) > std::numeric_limits<unsigned short>::max() ) ? std::numeric_limits<unsigned short>::max() : (UShort_t) ( TMath::Nint(beta * 20000.) );
  }
}
