//
//  The version of the class should be changed every time
//  when any changes in the codes are done
//  Grigory Nigmatkulov: 2016/12/15
//
#ifndef STFEMTOTRACK_H
#define STFEMTOTRACK_H

/// ROOT headers
#include <TObject.h>
#include <TVector3.h>
#include <TMath.h>

/// Headers for helix
#include "StPicoHelix.h"
#include "StPicoPhysicalHelix.h"
#include "SystemOfUnits.h"
#include "PhysicalConstants.h"

const Float_t __SCALE__ = 100.;
const Float_t __INV_SCALE__  = 0.01;
const UShort_t USHORTMAX = 65535;
const Short_t  SHORTMAX = 32767;

//_________________
class StFemtoTrack : public TObject {

 public:
  //Constructor
  StFemtoTrack();
  //Copy constructor
  StFemtoTrack(const StFemtoTrack &copy);
  //Destructor
  virtual ~StFemtoTrack();

  //Setters
  void SetId(Short_t id)               { mId = (UShort_t)id; }
  void SetNHits(Char_t nHits)          { mNHits = nHits; }           //nHits * charge
  void SetNHits(UShort_t nHits)        { mNHits = (Char_t)nHits; }   //nHits * charge
  void SetNHitsPoss(UShort_t nHits)    { mNHitsPoss = (UChar_t)nHits; }
  //void SetNHitsDedx(UShort_t nHits)    { mNHitsDedx = (UChar_t)nHits; }
  void SetNSigmaElectron(Float_t ns);
  void SetNSigmaPion(Float_t ns);
  void SetNSigmaKaon(Float_t ns);
  void SetNSigmaProton(Float_t ns);
  void SetDedx(Float_t dEdx);
  void SetDedx(Double_t dEdx);
  void SetMap0(Int_t map0)              { mMap0 = (UInt_t)map0; } //Set topology map
  void SetMap1(Int_t map1)              { mMap1 = (UInt_t)map1; } //Set topology map
  void SetPx(Float_t px)                { mMomX = px; } //Set px of the primary track
  void SetPy(Float_t py)                { mMomY = py; } //Set py of the primary track
  void SetPz(Float_t pz)                { mMomZ = pz; } //Set pz of the primary track
  void SetP(Float_t px, Float_t py, Float_t pz) { mMomX = px; mMomY = py; mMomZ = pz; }
  void SetOriginX(Float_t x)            { mOriginX = x; } //Set origin x at dca to primary vertex
  void SetOriginY(Float_t y)            { mOriginY = y; } //Set origin y at dca to primary vertex
  void SetOriginZ(Float_t z)            { mOriginZ = z; } //Set origin z at dca to primary vertex
  void SetOriginXYZ(Float_t x, Float_t y, Float_t z) { mOriginX = x; mOriginY = y; mOriginZ = z; }
  void SetGlobalPx(Float_t px)          { mGlobMomX = px; } //Set px of the global track
  void SetGlobalPy(Float_t py)          { mGlobMomY = py; } //Set py of the global track
  void SetGlobalPz(Float_t pz)          { mGlobMomZ = pz; } //Set pz of the global track
  void SetGlobalP(Float_t px, Float_t py, Float_t pz) { mGlobMomX = px; mGlobMomY = py; mGlobMomZ = pz; }
  void SetBeta(Float_t beta);

  //Getters
  Short_t   GetId() const                    { return (Short_t)mId; }
  Short_t   GetType() const;
  Bool_t    GetIsPrimary() const;
  Short_t   GetFlag() const                  { return 300; }
  UShort_t  GetNHits() const                 { return (UShort_t)TMath::Abs(mNHits); }
  UShort_t  GetNHitsFit() const              { return (UShort_t)TMath::Abs(mNHits); } 
  UShort_t  GetNHitsPoss() const             { return (UShort_t)mNHitsPoss; }
  //UShort_t  GetNHitsDedx() const             { return (UShort_t)mNHitsDedx; }
  UShort_t  GetNHitsDedx() const             { return (UShort_t)TMath::Abs(mNHits); } //Temporary
  Float_t   GetNSigmaElectron() const        { return (Float_t)mNSigmaElectron*__INV_SCALE__; }
  Float_t   GetNSigmaPion() const            { return (Float_t)mNSigmaPion*__INV_SCALE__; }
  Float_t   GetNSigmaKaon() const            { return (Float_t)mNSigmaKaon*__INV_SCALE__; }
  Float_t   GetNSigmaProton() const          { return (Float_t)mNSigmaProton*__INV_SCALE__; }
  Float_t   GetPidProbElectron() const       { return 0.5; }
  Float_t   GetPidProbPion() const           { return 0.5; }
  Float_t   GetPidProbKaon() const           { return 0.5; }
  Float_t   GetPidProbProton() const         { return 0.5; }
  Double_t  GetDedx() const                  { return (Double_t)mDedx * 1e-9; }
  UInt_t    GetMap0() const                  { return (UInt_t)mMap0; }
  UInt_t    GetMap1() const                  { return (UInt_t)mMap1; }
  Float_t   GetChi2() const                  { return 1.; }
  Short_t   GetCharge() const                { return (mNHits>0) ? +1 : -1; }
  Float_t   GetEta() const;
  
  Float_t   GetOriginX() const               { return mOriginX; }
  Float_t   GetOriginY() const               { return mOriginY; }
  Float_t   GetOriginZ() const               { return mOriginZ; }
  Float_t   GetDCAx(Float_t pVtxX) const;
  Float_t   GetDCAy(Float_t pVtxY) const;
  Float_t   GetDCAz(Float_t pVtxZ) const;
  Float_t   GetDCAxy(Float_t pVtxX, Float_t pVtxY) const;
  Float_t   GetDCAxyz(Float_t pVtxX, Float_t pVtxY, Float_t pVtxZ) const;
  Float_t   GetGlobalDCAx(Float_t pVtxX) const;
  Float_t   GetGlobalDCAy(Float_t pVtxY) const;
  Float_t   GetGlobalDCAz(Float_t pVtxZ) const;
  Float_t   GetGlobalDCAxy(Float_t pVtxX, Float_t pVtxY) const;
  Float_t   GetGlobalDCAxyz(Float_t pVtxX, Float_t pVtxY, Float_t pVtxZ) const;

  Float_t   GetPx() const    { return mMomX; }
  Float_t   GetPy() const    { return mMomY; }
  Float_t   GetPz() const    { return mMomZ; }
  TVector3  GetP() const;
  Float_t   GetPt() const;
  Float_t   GetPtot() const;
  Float_t   GetGlobalPx() const     { return mGlobMomX; }
  Float_t   GetGlobalPy() const     { return mGlobMomY; }
  Float_t   GetGlobalPz() const     { return mGlobMomZ; }
  TVector3  GetGlobalP() const;
  Float_t   GetGlobalPt() const;
  Float_t   GetGlobalPtot() const;
  Float_t   GetBeta() const         { return (mBeta>0) ? ( (Float_t)mBeta / 20000. ) : -999.; }
  Float_t   GetMassSqr() const;
  Bool_t    GetIsTofTrack();

  /// global momentum at point of DCA to pVtx, B should be in kilogauss
  TVector3 GetGMom(TVector3 const& pVtx, Float_t B) const;
  StPicoPhysicalHelix GetHelix(Float_t const B) const;

 private:

  UShort_t  mId;
  Char_t    mNHits;           //q * NHits
  UChar_t   mNHitsPoss;
  //UChar_t   mNHitsDedx;
  Short_t   mNSigmaElectron;  // ns*1000     !
  Short_t   mNSigmaPion;      // ns*1000     !
  Short_t   mNSigmaKaon;      // ns*1000     !
  Short_t   mNSigmaProton;    // ns*1000     !
  UShort_t  mDedx;            // dEdx*1e9    !
  UInt_t    mMap0;            // TopologyMap data0
  UInt_t    mMap1;            // TopologyMap data1
  Float_t   mMomX;            // Primary Px at pVtx
  Float_t   mMomY;            // Primary Py at pVtx
  Float_t   mMomZ;            // Primary Pz at pVtx
  Float_t   mOriginX;         // x at dca to the primary vertex
  Float_t   mOriginY;         // y at dca to the primary vertex
  Float_t   mOriginZ;         // z at dca to the primary vertex
  Float_t   mGlobMomX;        // Px Global at DCA to pVtx
  Float_t   mGlobMomY;        // Py Global at DCA to pVtx
  Float_t   mGlobMomZ;        // Pz Global at DCA to pVtx
  UShort_t  mBeta;            // TOF beta * 20000

  ClassDef(StFemtoTrack, 3)
};

/// Return the global momentum at the dca point to the pVtx (usually it is the primary vertex.   B - magnetic field from PicoEvent::bField()
//_________________
inline TVector3 StFemtoTrack::GetGMom(TVector3 const& pVtx, Float_t const B) const {
  StPicoPhysicalHelix gHelix = GetHelix(B);
  return gHelix.momentumAt( gHelix.pathLength(pVtx), B * kilogauss );
}

//_________________
inline StPicoPhysicalHelix StFemtoTrack::GetHelix(Float_t const B) const {
  return StPicoPhysicalHelix( TVector3(mGlobMomX, mGlobMomY, mGlobMomZ),
			      TVector3(mOriginX, mOriginY, mOriginZ),
			      B * kilogauss,
			      static_cast<float>( GetCharge() ) );
}

#endif
