#include "StHbtMaker/CorrFctn/QinvCorrFctnOpposHemisphere.h"
#include <sstream>
#include <cstdio>
#include <TRandom3.h>

#ifdef __ROOT__
ClassImp(QinvCorrFctnOpposHemisphere)
#endif

//_________________
QinvCorrFctnOpposHemisphere::QinvCorrFctnOpposHemisphere(char* title,
							 const int& nbins, 
							 const float& QinvLo, 
							 const float& QinvHi) {
   // set up numerator
  //  title = "Num Qinv (MeV/c)";
  stringstream TitNum,TitDen,TitRat;
  TitNum.str("");TitDen.str("");TitRat.str("");
  TitNum<<title<<"_Num";
  TitDen<<title<<"_Den";
  TitRat<<title<<"_Rat";
  mNumerator = new StHbt1DHisto(TitNum.str().c_str(),
				TitNum.str().c_str(),
				nbins,QinvLo,QinvHi);
  // set up denominator
  //title = "Den Qinv (MeV/c)";
  mDenominator = new StHbt1DHisto(TitDen.str().c_str(),
				  TitDen.str().c_str(),
				  nbins,QinvLo,QinvHi);
  // set up ratio
  //title = "Ratio Qinv (MeV/c)";
  mRatio = new StHbt1DHisto(TitRat.str().c_str(),
			    TitRat.str().c_str(),
			    nbins,QinvLo,QinvHi);
  // this next bit is unfortunately needed so that we can have many histos of same "title"
  // it is neccessary if we typedef StHbt1DHisto to TH1d (which we do)
  mNumerator->Sumw2();
  mNumerator->SetDirectory(0);
  mDenominator->Sumw2();
  mDenominator->SetDirectory(0);
  mRatio->Sumw2();
  mRatio->SetDirectory(0);

  mRandy = new TRandom3();
}

//_________________
QinvCorrFctnOpposHemisphere::~QinvCorrFctnOpposHemisphere() {
  delete mNumerator;
  delete mDenominator;
  delete mRatio;
}

//_________________
void QinvCorrFctnOpposHemisphere::Finish() {
  mRatio->Divide(mNumerator,mDenominator,1.0,1.0);
}

//_________________
void QinvCorrFctnOpposHemisphere::AddMixedPair(const StHbtPair* pair) {
  /*Empty*/
}
//_________________
void QinvCorrFctnOpposHemisphere::AddRealPair(const StHbtPair* pair) {

  double weight = 1.0;
  double Qinv = fabs(pair->qInv());
  mNumerator->Fill(Qinv);

  double RotatedQinv;
  //If mRandy>0.5 then rotate the first track of the pair.
  //In the other case rotate the second track of the pair.
  if(mRandy->Rndm()>0.5) {
      
    mTrack1.SetPxPyPzE(-pair->track1()->FourMomentum().x(),
		       -pair->track1()->FourMomentum().y(),
		       -pair->track1()->FourMomentum().z(),
		       pair->track1()->FourMomentum().e());

    mTrack2.SetPxPyPzE(pair->track2()->FourMomentum().x(),
		       pair->track2()->FourMomentum().y(),
		       pair->track2()->FourMomentum().z(),
		       pair->track2()->FourMomentum().e());
    mDiffVect = mTrack1 - mTrack2;
    RotatedQinv = -mDiffVect.M();

    /*
    cout << "Rotating the first particle: " << endl
	 << " px: " << pair->track1()->FourMomentum().x() << " -> " << mTrack1.Px() << endl
	 << " py: " << pair->track1()->FourMomentum().y() << " -> " << mTrack1.Py() << endl
	 << " pz: " << pair->track1()->FourMomentum().z() << " -> " << mTrack1.Pz() << endl
	 << " e:  " << pair->track1()->FourMomentum().e() << " -> " << mTrack1.E() << endl
	 << "Rotated Qinv: " << RotatedQinv << endl;
    */
  }
  else {
    
    mTrack1.SetPxPyPzE(pair->track1()->FourMomentum().x(),
		       pair->track1()->FourMomentum().y(),
		       pair->track1()->FourMomentum().z(),
		       pair->track1()->FourMomentum().e());

    mTrack2.SetPxPyPzE(-pair->track2()->FourMomentum().x(),
		       -pair->track2()->FourMomentum().y(),
		       -pair->track2()->FourMomentum().z(),
		       pair->track2()->FourMomentum().e());
    mDiffVect = mTrack1 - mTrack2;
    RotatedQinv = -mDiffVect.M();

    /*
    cout << "Rotating the second particle: " << endl
	 << " px: " << pair->track2()->FourMomentum().x() << " -> " << mTrack2.Px() << endl
	 << " py: " << pair->track2()->FourMomentum().y() << " -> " << mTrack2.Py() << endl
	 << " pz: " << pair->track2()->FourMomentum().z() << " -> " << mTrack2.Pz() << endl
	 << " e:  " << pair->track2()->FourMomentum().e() << " -> " << mTrack2.E() << endl
	 << "Rotated Qinv: " << RotatedQinv << endl;
    */
  }
  mDenominator->Fill(RotatedQinv,weight);
}

//____________________________
StHbtString QinvCorrFctnOpposHemisphere::Report(){
  string stemp = "Qinv Correlation Function of Opposite Hemi-sphere Report:\n";
  char ctemp[100];
  sprintf(ctemp,"Number of entries in numerator:\t%E\n",mNumerator->GetEntries());
  stemp += ctemp;
  sprintf(ctemp,"Number of entries in denominator:\t%E\n",mDenominator->GetEntries());
  stemp += ctemp;
  sprintf(ctemp,"Number of entries in ratio:\t%E\n",mRatio->GetEntries());
  stemp += ctemp;
  //  stemp += mCoulombWeight->Report();
  StHbtString returnThis = stemp;
  return returnThis;
}
