/*
 * Look at the opposite hemi-sphere.
 * Rotate one particle of the pair as follow (px,py,pz,e)->(-px,-py,-pz,e)
 * 2015/03/18 gnigmat
 */
#ifndef QinvCorrFctnOpposHemisphere_hh
#define QinvCorrFctnOpposHemisphere_hh

#include "StHbtMaker/Base/StHbtCorrFctn.hh"
#include "StHbtMaker/Base/StHbtPairCut.h"
#include <TRandom3.h>
#include <TLorentzVector.h>

class QinvCorrFctnOpposHemisphere : public StHbtCorrFctn {
 public:
  QinvCorrFctnOpposHemisphere(char* title,const int& nbins=50, 
			      const float& QinvLo=0.,const float& QinvHi=1.);
  virtual ~QinvCorrFctnOpposHemisphere();

  virtual StHbtString Report();
  virtual void AddRealPair(const StHbtPair*);
  virtual void AddMixedPair(const StHbtPair*);

  virtual void Finish();

  StHbt1DHisto* Numerator();
  StHbt1DHisto* Denominator();
  StHbt1DHisto* Ratio();

 private:
  StHbt1DHisto* mNumerator;
  StHbt1DHisto* mDenominator;
  StHbt1DHisto* mRatio;
  
  TRandom3 *mRandy;
  TLorentzVector mTrack1;
  TLorentzVector mTrack2;
  TLorentzVector mDiffVect;

#ifdef __ROOT__
  ClassDef(QinvCorrFctnOpposHemisphere, 1)
#endif
};

inline StHbt1DHisto* QinvCorrFctnOpposHemisphere::Numerator(){return mNumerator;}
inline StHbt1DHisto* QinvCorrFctnOpposHemisphere::Denominator(){return mDenominator;}
inline StHbt1DHisto* QinvCorrFctnOpposHemisphere::Ratio(){return mRatio;}
#endif //QinvCorrFctnOpposHemisphere_hh
