#include "StHbtMaker/CorrFctn/QinvCorrFctnRotationKt.h"
#include <sstream>
#include <cstdio>
#include <TRandom3.h>

#ifdef __ROOT__
ClassImp(QinvCorrFctnRotationKt)
#endif

//_________________
QinvCorrFctnRotationKt::QinvCorrFctnRotationKt(char* title,
					       const int& nbins,
					       const float& QinvLo, 
					       const float& QinvHi,
					       const int& nCFs,
					       const float& KtLo,
					       const float& KtHi) {
  
  mNumberCFs = nCFs;
  mKtMin = KtLo;
  mKtMax = KtHi;
  mDeltaKt = (mKtMax-mKtMin)/mNumberCFs;

  // set up numerators
  mNumerator = new StHbt1DHisto[mNumberCFs];

  // set up denominators
  mDenominator = new StHbt1DHisto[mNumberCFs];

  stringstream TitNum,TitDen,TitCurrent;
  TitNum.str("");TitDen.str("");
  TitNum<<title<<"_Num";
  TitDen<<title<<"_Den";

  for(int i=0; i<mNumberCFs; i++) {
    TitCurrent.str("");
    TitCurrent<<TitNum.str()<<"_bin_"<<i;
    mNumerator[i].SetName(TitCurrent.str().c_str());
    mNumerator[i].SetTitle(TitCurrent.str().c_str());
    mNumerator[i].SetBins(nbins,QinvLo,QinvHi);
    mNumerator[i].SetDirectory(0);
    mNumerator[i].Sumw2();

    TitCurrent.str("");
    TitCurrent<<TitDen.str()<<"_bin_"<<i;
    mDenominator[i].SetName(TitCurrent.str().c_str());
    mDenominator[i].SetTitle(TitCurrent.str().c_str());
    mDenominator[i].SetBins(nbins,QinvLo,QinvHi);
    mDenominator[i].SetDirectory(0);
    mDenominator[i].Sumw2();
  }

  mRandy = new TRandom3();
}

//_________________
QinvCorrFctnRotationKt::~QinvCorrFctnRotationKt() {
  delete [] mNumerator;
  delete [] mDenominator;
}

//_________________
void QinvCorrFctnRotationKt::WriteOutHistos() {
  
  for(int i=0; i<mNumberCFs; i++) {
    mNumerator[i].Write();
    mDenominator[i].Write();
  }
}

//_________________
void QinvCorrFctnRotationKt::Finish() {
  /* Ratios might be here*/
}

//_________________
void QinvCorrFctnRotationKt::AddMixedPair(const StHbtPair* pair) {
  /*Empty*/
}

//_________________
void QinvCorrFctnRotationKt::AddRealPair(const StHbtPair* pair) {

  double weight = 1.0;
  int mIndex = (int)(((fabs(pair->kT())-mKtMin)/mDeltaKt));
  if ((mIndex>=0)&&(mIndex<mNumberCFs)) {
    mNumerator[mIndex].Fill(fabs(pair->qInv()),1.0);
  }

  double RotatedQinv;
  //If mRandy>0.5 then rotate the first track of the pair.
  //In the other case rotate the second track of the pair.
  if(mRandy->Rndm()>0.5) {
      
    mTrack1.SetPxPyPzE(-pair->track1()->FourMomentum().x(),
		       -pair->track1()->FourMomentum().y(),
		       pair->track1()->FourMomentum().z(),
		       pair->track1()->FourMomentum().e());

    mTrack2.SetPxPyPzE(pair->track2()->FourMomentum().x(),
		       pair->track2()->FourMomentum().y(),
		       pair->track2()->FourMomentum().z(),
		       pair->track2()->FourMomentum().e());
    mDiffVect = mTrack1 - mTrack2;
    mSumVect = mTrack1 + mTrack2;
    RotatedQinv = -mDiffVect.M();

    /*
    cout << "Rotating the first particle: " << endl
	 << " px: " << pair->track1()->FourMomentum().x() 
	 << " -> " << mTrack1.Px() << endl
	 << " py: " << pair->track1()->FourMomentum().y() 
	 << " -> " << mTrack1.Py() << endl
	 << " pz: " << pair->track1()->FourMomentum().z() 
	 << " -> " << mTrack1.Pz() << endl
	 << " e:  " << pair->track1()->FourMomentum().e() 
	 << " -> " << mTrack1.E() << endl
	 << "Rotated Qinv: " << RotatedQinv << endl;
    */
  }
  else {
    
    mTrack1.SetPxPyPzE(pair->track1()->FourMomentum().x(),
		       pair->track1()->FourMomentum().y(),
		       pair->track1()->FourMomentum().z(),
		       pair->track1()->FourMomentum().e());

    mTrack2.SetPxPyPzE(-pair->track2()->FourMomentum().x(),
		       -pair->track2()->FourMomentum().y(),
		       pair->track2()->FourMomentum().z(),
		       pair->track2()->FourMomentum().e());
    mDiffVect = mTrack1 - mTrack2;
    mSumVect = mTrack1 + mTrack2;
    RotatedQinv = -mDiffVect.M();

    /*
    cout << "Rotating the second particle: " << endl
	 << " px: " << pair->track2()->FourMomentum().x() 
	 << " -> " << mTrack2.Px() << endl
	 << " py: " << pair->track2()->FourMomentum().y() 
	 << " -> " << mTrack2.Py() << endl
	 << " pz: " << pair->track2()->FourMomentum().z() 
	 << " -> " << mTrack2.Pz() << endl
	 << " e:  " << pair->track2()->FourMomentum().e() 
	 << " -> " << mTrack2.E() << endl
	 << "Rotated Qinv: " << RotatedQinv << endl;
    */
  }

  //I think that after the rotation was done
  //we have to recalculate transverse pair momentum
  //and out qInv to the different kT bin (if it is different)
  mIndex = (int)( (mSumVect.Pt()/2-mKtMin)/mDeltaKt );
  if((mIndex>=0)&&(mIndex<mNumberCFs)) {
    mDenominator[mIndex].Fill(RotatedQinv,weight);
  }
}

//_________________
StHbtString QinvCorrFctnRotationKt::Report(){
  int mNumeratorEntries=0;
  int mDenominatorEntries=0;

  for (int i=0; i<mNumberCFs; i++){
    mNumeratorEntries += (int)mNumerator[i].GetEntries();
    mDenominatorEntries += (int)mDenominator[i].GetEntries();
  }
  string stemp = "Qinv Correlation Function of Rotated Particles Report:\n";
  char ctemp[100];
  sprintf(ctemp,"Number of entries in all numerators:\t%i\n",mNumeratorEntries);
  stemp += ctemp;
  sprintf(ctemp,"Number of entries in all denominators:\t%i\n",mDenominatorEntries);
  stemp += ctemp;
  StHbtString returnThis = stemp;
  return returnThis;
}
