#include "StHbtMaker/Infrastructure/StHbtVertexMultSphAnalysis.h"
#include "StHbtMaker/Infrastructure/StHbtParticleCollection.hh"
#include "StHbtMaker/Base/StHbtTrackCut.h"
#include "StHbtMaker/Base/StHbtV0Cut.h"
#include "StHbtMaker/Base/StHbtKinkCut.h"
#include "StHbtMaker/Infrastructure/StHbtPicoEventCollectionVector.hh"
#include "StHbtMaker/Infrastructure/StHbtPicoEventCollectionVectorHideAway.hh"


#ifdef __ROOT__
ClassImp(StHbtVertexMultSphAnalysis)
#endif

extern void FillHbtParticleCollection(StHbtParticleCut*         partCut,
                                      StHbtEvent*               hbtEvent,
                                      StHbtParticleCollection*  partCollection);


//____________________________
StHbtVertexMultSphAnalysis::StHbtVertexMultSphAnalysis(unsigned int binsVertex, double minVertex, double maxVertex,
                                                       unsigned int binsMult,   double minMult,   double maxMult,
                                                       unsigned int binsSph,    double minSph,    double maxSph) 
    : mVertexZBins(binsVertex), mMultBins(binsMult), mSphBins(binsSph) {

    //  mControlSwitch     = 0;
    mEventCut           = 0;
    mFirstParticleCut   = 0;
    mSecondParticleCut  = 0;
    mPairCut            = 0;
    mCorrFctnCollection = new StHbtCorrFctnCollection;

    mVertexZ[0]       = minVertex;
    mVertexZ[1]       = maxVertex;
    mUnderFlowVertexZ = 0;
    mOverFlowVertexZ  = 0;

    mMult[0] = minMult;
    mMult[1] = maxMult;
    mUnderFlowMult = 0;
    mOverFlowMult = 0;

    mSph[0] = minSph;
    mSph[1] = maxSph;
    mUnderFlowSph = 0;
    mOverFlowSph = 0;

    if (mMixingBuffer) delete mMixingBuffer;
    mPicoEventCollectionVectorHideAway = new StHbtPicoEventCollectionVectorHideAway(mVertexZBins,mVertexZ[0],mVertexZ[1],
                                                                                    mMultBins,mMult[0],mMult[1],
                                                                                    mSphBins, mSph[0],mSph[1]);
}

//____________________________
StHbtVertexMultSphAnalysis::StHbtVertexMultSphAnalysis(const StHbtVertexMultSphAnalysis &a) : StHbtAnalysis() {
    //StHbtVertexMultSphAnalysis();
    mEventCut           = 0;
    mFirstParticleCut   = 0;
    mSecondParticleCut  = 0;
    mPairCut            = 0;
    mCorrFctnCollection = new StHbtCorrFctnCollection;

    mVertexZBins      = a.mVertexZBins;
    mVertexZ[0]       = a.mVertexZ[0];
    mVertexZ[1]       = a.mVertexZ[1];
    mUnderFlowVertexZ = 0;
    mOverFlowVertexZ  = 0;

    mMultBins      = a.mMultBins;
    mMult[0]       = a.mMult[0];
    mMult[1]       = a.mMult[1];
    mUnderFlowMult = 0;
    mOverFlowMult  = 0;

    mSphBins      = a.mSphBins;
    mSph[0]       = a.mSph[0];
    mSph[1]       = a.mSph[1];
    mUnderFlowSph = 0;
    mOverFlowSph  = 0;

    if (mMixingBuffer) delete mMixingBuffer;
    mPicoEventCollectionVectorHideAway = new StHbtPicoEventCollectionVectorHideAway(mVertexZBins,mVertexZ[0],mVertexZ[1],
                                                                                    mMultBins,mMult[0],mMult[1],
                                                                                    mSphBins,mSph[0],mSph[1]);

    // find the right event cut
    mEventCut = a.mEventCut->Clone();
    // find the right first particle cut
    mFirstParticleCut = a.mFirstParticleCut->Clone();
    // find the right second particle cut
    if (a.mFirstParticleCut==a.mSecondParticleCut)
        SetSecondParticleCut(mFirstParticleCut); // identical particle hbt
    else
        mSecondParticleCut = a.mSecondParticleCut->Clone();

    mPairCut = a.mPairCut->Clone();

    if ( mEventCut ) {
        SetEventCut(mEventCut); // this will set the myAnalysis pointer inside the cut
        cout << " StHbtVertexMultSphAnalysis::StHbtVertexMultSphAnalysis(const StHbtVertexMultSphAnalysis& a) - event cut set " << endl;
    }
    if ( mFirstParticleCut ) {
        SetFirstParticleCut(mFirstParticleCut); // this will set the myAnalysis pointer inside the cut
        cout << " StHbtVertexMultSphAnalysis::StHbtVertexMultSphAnalysis(const StHbtVertexMultSphAnalysis& a) - first particle cut set " << endl;
    }
    if ( mSecondParticleCut ) {
        SetSecondParticleCut(mSecondParticleCut); // this will set the myAnalysis pointer inside the cut
        cout << " StHbtVertexMultSphAnalysis::StHbtVertexMultSphAnalysis(const StHbtVertexMultSphAnalysis& a) - second particle cut set " << endl;
    }  if ( mPairCut ) {
        SetPairCut(mPairCut); // this will set the myAnalysis pointer inside the cut
        cout << " StHbtVertexMultSphAnalysis::StHbtVertexMultSphAnalysis(const StHbtVertexMultSphAnalysis& a) - pair cut set " << endl;
    }

    StHbtCorrFctnIterator iter;
    for (iter=a.mCorrFctnCollection->begin(); iter!=a.mCorrFctnCollection->end(); iter++) {
        cout << " StHbtVertexMultSphAnalysis::StHbtVertexMultSphAnalysis(const StHbtVertexMultSphAnalysis& a) - looking for correlation functions " << endl;
        StHbtCorrFctn *fctn = (*iter)->Clone();
        if (fctn) AddCorrFctn(fctn);
        else cout << " StHbtVertexMultSphAnalysis::StHbtVertexMultSphAnalysis(const StHbtVertexMultSphAnalysis& a) - correlation function not found " << endl;
    }

    mNumEventsToMix = a.mNumEventsToMix;

    cout << " StHbtVertexMultSphAnalysis::StHbtVertexMultSphAnalysis(const StHbtVertexMultSphAnalysis& a) - analysis copied " << endl;
}

//____________________________
StHbtVertexMultSphAnalysis::~StHbtVertexMultSphAnalysis() {
    // now delete every PicoEvent in the EventMixingBuffer and then the Buffer itself
    delete mPicoEventCollectionVectorHideAway;
}

//____________________________
StHbtString StHbtVertexMultSphAnalysis::Report() {
    cout << "StHbtVertexMultSphAnalysis - constructing Report..."<<endl;
    char Ctemp[200];
    StHbtString temp = "-----------\nHbt StHbtVertexMultSphAnalysis Report:\n";

    sprintf(Ctemp,"Events are mixed in %d VertexZ bins in the range %E cm to %E cm\n",mVertexZBins,mVertexZ[0],mVertexZ[1]);
    temp += Ctemp;
    sprintf(Ctemp,"Events underflowing: %d\n",mUnderFlowVertexZ);
    temp += Ctemp;
    sprintf(Ctemp,"Events overflowing: %d\n",mOverFlowVertexZ);
    temp += Ctemp;

    sprintf(Ctemp,"Events are mixed in %d Mult bins in the range %E to %E\n",mMultBins,mMult[0],mMult[1]);
    temp += Ctemp;
    sprintf(Ctemp,"Events underflowing: %d\n",mUnderFlowMult);
    temp += Ctemp;
    sprintf(Ctemp,"Events overflowing: %d\n",mOverFlowMult);
    temp += Ctemp;

    sprintf(Ctemp,"Events are mixed in %d Sphericity bins in the range %E to %E\n",mSphBins,mSph[0],mSph[1]);
    temp += Ctemp;
    sprintf(Ctemp,"Events underflowing: %d\n",mUnderFlowSph);
    temp += Ctemp;
    sprintf(Ctemp,"Events overflowing: %d\n",mOverFlowSph);
    temp += Ctemp;

    sprintf(Ctemp,"Now adding StHbtAnalysis(base) Report\n");
    temp += Ctemp;
    temp += StHbtAnalysis::Report();
    StHbtString returnThis=temp;
    return returnThis;
}

//_________________________
void StHbtVertexMultSphAnalysis::ProcessEvent(const StHbtEvent *hbtEvent) {
    // get right mixing buffer
    double vertexZ = hbtEvent->PrimVertPos().z();
    double mult    = hbtEvent->UncorrectedNumberOfPrimaries();
    double sph     = (double)hbtEvent->Sphericity();
    mMixingBuffer  = mPicoEventCollectionVectorHideAway->PicoEventCollection(vertexZ,mult,sph);
    if (!mMixingBuffer) {
        if ( vertexZ < mVertexZ[0] ) mUnderFlowVertexZ++;
        if ( vertexZ > mVertexZ[1] ) mOverFlowVertexZ++;

        if ( mult < mMult[0] ) mUnderFlowMult++;
        if ( mult > mMult[1] ) mOverFlowMult++;

        if ( sph < mSph[0] ) mUnderFlowSph++;
        if ( sph > mSph[1] ) mOverFlowSph++;

        return;
    }
    // call ProcessEvent() from StHbtAnalysis-base
    StHbtAnalysis::ProcessEvent(hbtEvent);
}
